#include "blocklib.h"
#include <stdlib.h>
#include <time.h>

char* XORencryptor(const char data[8], const char* const key)
{       char* retval=new char[8];

        for (unsigned char i=0;i<8;i++)
                retval[i]=data[i]^key[i];

        return retval;
}

void ECB_Encrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char* cypherblock;
        unsigned char position;

        srand(time(NULL));

        position=0;
        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock=cipher(block,key);
                        output.write(cypherblock,8);
                        delete cypherblock;

                        position=0;
                }
                block[position]=input.get();
        }

        for (unsigned char i=position;i<7;i++)
                block[i]=rand()%256;

        block[7]=position;

        cypherblock=cipher(block,key);
        output.write(cypherblock,8);
        delete cypherblock;
}

void ECB_Decrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char* cypherblock;
        unsigned char position;
        char peekchar;

        position=0;

        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock=cipher(block,key);

                        peekchar=input.get();
                        if (input.good())
                        {       output.write(cypherblock,8);
                                block[0]=peekchar;
                                position=0;
                        }
                        else
                        {       output.write(cypherblock,cypherblock[7]);
                        }
                        delete cypherblock;
                }
                else
                        block[position]=input.get();
        }
}

void CBC_Encrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char lastblock[8];
        char* cypherblock1;
        char* cypherblock2;
        unsigned char position;

        srand(time(NULL));

        for (position=0;position<8;position++)
                lastblock[position]=rand()%256;
        output.write(lastblock,8);

        position=0;

        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock1=XORencryptor(lastblock,block);
                        cypherblock2=cipher(cypherblock1,key);

                        for (position=0;position<8;position++)
                                lastblock[position]=cypherblock2[position];

                        output.write(lastblock,8);
                        delete cypherblock1;
                        delete cypherblock2;

                        position=0;
                }
                block[position]=input.get();
        }

        for (unsigned char i=position;i<7;i++)
                block[i]=rand()%256;
        block[7]=position;

        cypherblock1=XORencryptor(lastblock,block);
        cypherblock2=cipher(cypherblock1,key);

        output.write(cypherblock2,8);
        delete cypherblock1;
        delete cypherblock2;
}

void CBC_Decrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char lastblock[8];
        char* cypherblock1;
        char* cypherblock2;
        unsigned char position;
        char peekchar;

        for (position=0;position<8;position++)
        {       lastblock[position]=input.get();
        }

        position=0;
        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock1=cipher(block,key);
                        cypherblock2=XORencryptor(lastblock,cypherblock1);

                        peekchar=input.get();
                        if (input.good())
                        {       output.write(cypherblock2,8);

                                for (position=0;position<8;position++)
                                        lastblock[position]=block[position];

                                block[0]=peekchar;
                                position=0;
                        }
                        else
                        {       output.write(cypherblock2,cypherblock2[7]);
                        }
                        delete cypherblock1;
                        delete cypherblock2;
                }
                else
                        block[position]=input.get();
        }
}

void CFB_Encrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char lastblock[8];
        char* cypherblock1;
        char* cypherblock2;
        unsigned char position;

        srand(time(NULL));

        for (position=0;position<8;position++)
                lastblock[position]=rand()%256;
        output.write(lastblock,8);

        position=0;

        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock1=cipher(lastblock,key);
                        cypherblock2=XORencryptor(block,cypherblock1);

                        for (position=0;position<8;position++)
                                lastblock[position]=cypherblock2[position];

                        output.write(cypherblock2,8);
                        delete cypherblock1;
                        delete cypherblock2;

                        position=0;
                }
                block[position]=input.get();
        }

        block[position]=0;
        if (position!=0)
        {       cypherblock1=cipher(lastblock,key);
                cypherblock2=XORencryptor(block,cypherblock1);

                output.write(cypherblock2,position);
                delete cypherblock1;
                delete cypherblock2;
        }
}

void CFB_Decrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char lastblock[8];
        char* cypherblock1;
        char* cypherblock2;
        unsigned char position;

        for (position=0;position<8;position++)
        {       lastblock[position]=input.get();
        }

        position=0;

        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock1=cipher(lastblock,key);
                        cypherblock2=XORencryptor(block,cypherblock1);

                        for (position=0;position<8;position++)
                                lastblock[position]=block[position];

                        output.write(cypherblock2,8);
                        delete cypherblock1;
                        delete cypherblock2;

                        for (position=0;position<8;position++)
                                block[position]=0;
                        position=0;
                }
                block[position]=input.get();
        }

        block[position]=0;
        if (position!=0)
        {       cypherblock1=cipher(lastblock,key);
                cypherblock2=XORencryptor(block,cypherblock1);

                output.write(cypherblock2,position);
                delete cypherblock1;
                delete cypherblock2;
        }
}

void OFB_Encrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char lastblock[8];
        char* cypherblock1;
        char* cypherblock2;
        unsigned char position;

        srand(time(NULL));

        for (position=0;position<8;position++)
                lastblock[position]=rand()%256;
        output.write(lastblock,8);

        position=0;

        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock1=cipher(lastblock,key);
                        cypherblock2=XORencryptor(block,cypherblock1);

                        for (position=0;position<8;position++)
                                lastblock[position]=cypherblock1[position];

                        output.write(cypherblock2,8);
                        delete cypherblock1;
                        delete cypherblock2;
                        position=0;
                }
                block[position]=input.get();
        }

        if (position!=0)
        {       cypherblock1=cipher(lastblock,key);
                cypherblock2=XORencryptor(block,cypherblock1);

                output.write(cypherblock2,position);
                delete cypherblock1;
                delete cypherblock2;
        }
}

void OFB_Decrypt (istream& input,ostream& output,const char* const key,char* (*cipher) (const char[8],const char* const))
{       char block[8];
        char lastblock[8];
        char* cypherblock1;
        char* cypherblock2;
        unsigned char position;

        for (position=0;position<8;position++)
        {       lastblock[position]=input.get();
        }

        position=0;

        block[position]=input.get();
        while (input.good())
        {       position++;
                if (position==8)
                {       cypherblock1=cipher(lastblock,key);
                        cypherblock2=XORencryptor(block,cypherblock1);

                        for (position=0;position<8;position++)
                                lastblock[position]=cypherblock1[position];

                        output.write(cypherblock2,8);
                        delete cypherblock1;
                        delete cypherblock2;

                        position=0;
                }
                block[position]=input.get();
        }

        block[position]=0;
        if (position!=0)
        {       cypherblock1=cipher(lastblock,key);
                cypherblock2=XORencryptor(block,cypherblock1);

                output.write(cypherblock2,position);
                delete cypherblock1;
                delete cypherblock2;
        }
}
